/*
    Task: ex1-show-inventory.c
    Author: James Mulligan
    Date: 31/10/2024
    Description: Takes the input from the command line in the given format.
        Then outputs it in the given format.

    Solution: 
        - Define our product struct
        - Parse through the input arguments and create new products and add them to our products array
        - Finally, output all the products in the given format.
*/

// include necessary libraries
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// typedefs
typedef struct Product Product;


// declare structs
struct Product
{
    char name[20];
    unsigned stock;
    float price;
    int discount;
};

/*
    The struct outlines the following:
    - name (product name) a string with max 20 chars
    - stock positive int representing no. of items available
    - price (float)
    - discount a binary (im using integer) value (1 = discount available, 0 = no discount)
*/

// function prototypes
void parseInput(int length, char*argv[], Product *products);
void generateOutput(int items, Product *products);

// main function
int main(int argc, char*argv[])
{
    int length = argc - 1;
    // this will give us the number of items we will have
    int items = length / 4;

    // initialise array of products
    Product products[100];

    // run functions
    parseInput(length, argv, products);
    generateOutput(items, products);

    // exit with code 0 if all okay
    return 0;
}


// actual function implementations
void parseInput(int length, char*argv[], Product *products)
{
    // two variables here, i to keep track of where we are in the input args,
    // pos to keep track of where we are in our products array
    // add 4 onto i each time to move onto the next product in input
    for (int i = 0, pos = 0; i < length; i+=4, pos++)
    {
        // will use pointers here to manipulate values in the struct
        Product *newProduct;
        // assign our new product a location in the products array
        newProduct = &products[pos];
        // adding information, changing data type as needed.
        strcpy(newProduct->name, argv[i + 1]);
        newProduct->stock = atoi(argv[i + 2]);
        newProduct->price = atof(argv[i + 3]);
        newProduct->discount = atoi(argv[i + 4]);
    }
}

// function to output in desired format.
void generateOutput(int items, Product *products)
{
    for (int i = 0; i < items; i++)
    {
        // get current products address
        Product *currentProduct = &products[i];

        char discount[] = "No Discount";
        // check if there is a discount on the product
        if (currentProduct->discount == 1)
        {
            // if so ammend discount value
            strcpy(discount, "Discounted");
        }
        // output the information
        printf("%s,%d,%.2f,%s\n", currentProduct->name, currentProduct->stock, currentProduct->price, discount);
    }
}