/*
    Task: ex2-array-degree.c
    Author: James Mulligan
    Date: 28/11/2024
    Description: Find the degree of a list of numbers

    Solution: 
        - Generate a linked list using the command line arguments
        - Iterate through each value in the linked list and find the occurences of each one
        - Keep track of the highest number of occurences
        - Output this number
*/

// include necessary libraries
#include <stdio.h>
#include <stdlib.h>

// typedefs
typedef struct Node Node;

// declare structs
struct Node {
    int value;
    Node* next;
};

// function prototypes
Node* generateLinkedList(int length, char*argv[]);
int countOccurences(Node *firstNode, int target);
int findDegree(Node *firstNode);

// main function
int main(int argc, char*argv[])
{
    int length = argc - 1;
    Node* pHeadNode = NULL;
    // generate our linked list
    pHeadNode = generateLinkedList(length, argv);

    // now find the degree
    int degree = findDegree(pHeadNode);
    printf("%d\n", degree);

    // exit with code 0 if all okay
    return 0;
}

// actual function implementations
// this function will generate a linked list starting at the head node and parsing cli args
Node* generateLinkedList(int length, char*argv[])
{
    Node *firstNode, *currentNode;
    // set the first node
    firstNode = (Node*)calloc(1, sizeof(Node));
    if (!firstNode)
    {
        printf("Failed to allocate memory! Exiting...\n");
        exit(1);
    }
    // set the current node to be the first node
    currentNode = firstNode;
    currentNode->value = atoi(argv[1]);

    // iterate through the remaining 
    for (int i = 1; i < length; i++)
    {
        // grab value from command line args
        int value = atoi(argv[i + 1]);

        // allocate some memory for the new node to add
        currentNode->next = (Node*)calloc(1, sizeof(Node));
        if (!currentNode->next)
        {
            printf("Failed to allocate memory! Exiting...\n");
            exit(1);
        }
        // move the current node to the next one in the list
        currentNode = currentNode->next;

        // edit the value
        currentNode->value = value;
    }
    
    // set the last node next to NULL
    currentNode->next = NULL;

    // return the head node at the end
    return firstNode;
}

// this function counts the number of times the target appears in the list
int countOccurences(Node *firstNode, int target)
{
    int count = 0;
    // set node initially to null
    Node* node = NULL;
    // iterate through every node.next until it hits null (the end)
    for (node = firstNode; node != NULL; node = node->next)
    {
        // if the value is our target, increase our count by 1
        if (node-> value == target)
        {
            count++;
        }
    }

    // return the count at the end
    return count;
}

// function which finds the number which appears most in linked list
int findDegree(Node *firstNode)
{
    int currDegree = -1;

    Node* node = NULL;
    for (node = firstNode; node != NULL; node = node->next)
    {
        // count num of occurences for each value in list
        int occurs = countOccurences(firstNode, node->value);
        if (occurs > currDegree)
        {
            currDegree = occurs;
        }
    }

    // return the final degree
    return currDegree;
}