/*
    Task: ex2-library-catalog.c
    Author: James Mulligan
    Date: 28/11/2024
    Description: Library Catalog System

    Solution: 
        - Use a doubly linked list
        - Parse the input and generate a linked list filled with the books and book info
        - Iterate through this in reverse to print the books in the given format in reverse
        - Then get the last argument and print books published after that year
*/

// include necessary libraries
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// typedefs
typedef struct Node Node;
typedef struct Book Book;

// declare structs
struct Book {
    char name[100];
    char author[100];
    int year;
};

// need to use a doubly linked list to print in reverse
struct Node {
    Book book;
    Node* next;
    Node* prev;
};

// function prototypes
Node* generateCatalog(int length, char*argv[]);
void printCatalog(Node* pLastNode);
void printAfterYear(Node* pLastNode, int yearFilter);

// main function
int main(int argc, char*argv[])
{
    int bookCount = atoi(argv[1]);
    Node* pLastNode = NULL;
    pLastNode = generateCatalog(bookCount, argv);

    printCatalog(pLastNode);

    // print line break
    printf("\n");

    // get the last command line arg
    int filterYear = atoi(argv[argc - 1]);
    printAfterYear(pLastNode, filterYear);

    // exit with code 0 if all okay
    return 0;
}

// actual function implementations
Node* generateCatalog(int length, char*argv[])
{
    Node *firstNode, *currentNode;

    // set the first node
    firstNode = (Node*)calloc(1, sizeof(Node));
    if (!firstNode)
    {
        printf("Failed to allocate memory! Exiting...\n");
        exit(1);
    }
    // set the current node to be the first node
    currentNode = firstNode;
    currentNode->prev = NULL;

    // iterate through command line args, but start every iteration on the book name index
    for (int i = 2; i < length * 3; i+=3)
    {
        // use string copy to edit values in struct
        strcpy(currentNode->book.name, argv[i]);
        strcpy(currentNode->book.author, argv[i + 1]);

        currentNode->book.year = atoi(argv[i + 2]);
        
        // if there are more elements to be added...
        if (i < (length * 3) - 1)
        {
            // allocate some memory for the new node to add
            currentNode->next = (Node*)calloc(1, sizeof(Node));
            if (!currentNode->next)
            {
                printf("Failed to allocate memory! Exiting...\n");
                exit(1);
            }
            // move the current node to the next one in the list
            currentNode->next->prev = currentNode;
            currentNode = currentNode->next;
        }
    }
    
    // set the last next node to NULL
    currentNode->next = NULL;

    // return the tail node at the end
    return currentNode;
}

// this function will iterate through the linked list to print the catalog
void printCatalog(Node* pLastNode)
{
    printf("Library Catalog:\n");

    Node* node = NULL;
    // iterate backwards through the list, starting at tail and going to beginning
    for (node = pLastNode; node != NULL; node = node->prev)
    {
        printf("Title: %s, Author: %s, Year: %d\n", node->book.name, node->book.author, node->book.year);
    }
}

// prints books from catalog only published after the given year
void printAfterYear(Node* pLastNode, int yearFilter)
{
    printf("Books published after %d:\n", yearFilter);

    // keep track if anything was found, so we can print nothing found message
    int found = 0; // 0 nothing found, 1 something was found

    Node* node = NULL;
    // iterate backwards through the list, starting at tail and going to beginning
    for (node = pLastNode; node != NULL; node = node->prev)
    {
        if (node->book.year > yearFilter)
        {
            printf("Title: %s, Author: %s, Year: %d\n", node->book.name, node->book.author, node->book.year);
            found = 1;
        }
    }
    // if nothing found, print message
    if (found == 0)
    {
        printf("No books found published after %d.\n", yearFilter);
    }
}