/*
    Task: ex2-sparse-matrix.c
    Author: James Mulligan
    Date: 28/11/2024
    Description: Generate a sparse matrix using the arguments, print it and find the sum of the main diagonal

    Solution: 
        - Parse the input
        - Generate a linked list of all non-zero elements
        - Print the list, putting 0's where there are no values
        - Then find the sum of the main diagonal and output this too
*/

// include necessary libraries
#include <stdio.h>
#include <stdlib.h>

// typedefs
typedef struct Node Node;

// declare structs
struct Node {
    int value;
    int posX;
    int posY;
    Node* next;
};

// function prototypes
Node* generateLinkedList(int count, char*argv[]);
void printMatrix(int rows, int cols, Node* pHeadNode);
int findValueAtPos(Node* pFirstNode, int targetX, int targetY);
int calculateDiagSum(Node* pFirstNode, int rows, int cols);

// main function
int main(int argc, char*argv[])
{
    int rows = atoi(argv[1]);
    int cols = atoi(argv[2]);
    int count = atoi(argv[3]);

    int length = argc - 1;
    Node* pHeadNode = NULL;

    // generate our linked list
    pHeadNode = generateLinkedList(count, argv);

    printMatrix(rows, cols, pHeadNode);

    // find the sum of the main diagonal here
    int sumOfDiagonal = calculateDiagSum(pHeadNode, rows, cols);
    printf("Sum of main diagonal: %d\n", sumOfDiagonal);

    // exit with code 0 if all okay
    return 0;
}

// actual function implementations
Node* generateLinkedList(int count, char*argv[])
{
    Node *firstNode, *currentNode;

    // set the first node
    firstNode = (Node*)calloc(1, sizeof(Node));
    if (!firstNode)
    {
        printf("Failed to allocate memory! Exiting...\n");
        exit(1);
    }
    // set the current node to be the first node
    currentNode = firstNode;

    // iterate through command line args, but start every iteration on the value
    // or could say, skipping the posititional arguments
    for (int i = 4; i <= (count * 3) + 1; i+=3)
    {
        // grab the information from the command line
        currentNode->value = atoi(argv[i]);
        currentNode->posX = atoi(argv[i + 1]);
        currentNode->posY = atoi(argv[i + 2]);

        // if there are more elements to be added...
        if (i < (count * 3) - 1)
        {
            // allocate some memory for the new node to add
            currentNode->next = (Node*)calloc(1, sizeof(Node));
            if (!currentNode->next)
            {
                printf("Failed to allocate memory! Exiting...\n");
                exit(1);
            }
            // move the current node to the next one in the list
            currentNode = currentNode->next;
        }
    }
    
    // set the last next node to NULL
    currentNode->next = NULL;

    // return the tail node at the end
    return firstNode;
}

void printMatrix(int rows, int cols, Node* pHeadNode)
{
    // go through all rows
    for (int i = 0; i < rows; i++)
    {
        // i is our current row
        // go through all columns
        for (int j = 0; j < cols; j++)
        {
            // j is our current column
            // check if there is a value at the current position in the matrix
            int curr = findValueAtPos(pHeadNode, i, j);
            // this function returns the value if found, else will return 0
            printf("%d ", curr);
        }
        printf("\n");
    }
}

// this function will return a value at a specified position if exists
int findValueAtPos(Node* pFirstNode, int targetX, int targetY)
{
    Node* node = NULL;
    // iterate through linked list untill the end is hit
    for (node = pFirstNode; node != NULL; node = node->next)
    {
        // if the current x and y position are equal to the target, we found what we are looking for!
        if (node->posX == targetX && node->posY == targetY)
        {
            // return it
            return node->value;
        }
    }

    // nothing was found, return 0
    return 0;
}

int calculateDiagSum(Node* pFirstNode, int rows, int cols)
{
    int sum = 0;
    for (int i = 0; i < rows; i++)
    {
        // i is our current row
        for (int j = 0; j < cols; j++)
        {
            // j is our current column
            // if i and j are the same we are on the main diagonal!
            if (i == j)
            {
                // find the value here and add it onto our sum
                sum += findValueAtPos(pFirstNode, i, j);
            }
        }
    }
    // return the sum at the end
    return sum;
}